#!/bin/bash

set -e -o pipefail

# General Java options
JAVA_OPTS+=" -showversion"
JAVA_OPTS+=" -XX:+PrintFlagsFinal"
JAVA_OPTS+=" -XX:+PrintCommandLineFlags"
JAVA_OPTS+=" -XX:+PrintStringTableStatistics"
JAVA_OPTS+=" -XX:ErrorFile=/tmp/error_java_identity_%p.log"
JAVA_OPTS+=" -Xlog:gc*:/tmp/migration.txt:time,level,tags:filesize=50M,filecount=1"

# memory options
# use parallel GC if no GC is added in JAVA_OPTS - the default G1 uses too much off-heap memory
if ! grep -E 'UseG1GC|UseConcMarkSweepGC|UseEpsilonGC|UseParallelGC|UseSerialGC|UseZGC' <<< "$JAVA_OPTS" > /dev/null; then
  echo "Setting GC to ParallelGC"
  JAVA_OPTS+=" -XX:+UseParallelGC"
fi

JAVA_OPTS+=" -Xms${JVM_XMS:-3712m} -Xmx${JVM_XMX:-3712m}"

echo "Setting metaspace to ${JVM_METASPACE:=256m}"
JAVA_OPTS+=" -XX:MaxMetaspaceSize=$JVM_METASPACE"

JAVA_OPTS+=" -Xss384k"
JAVA_OPTS+=" -XX:MaxJavaStackTraceDepth=0"
JAVA_OPTS+=" -XX:SurvivorRatio=8"
JAVA_OPTS+=" -XX:TargetSurvivorRatio=90"
JAVA_OPTS+=" -XX:MaxTenuringThreshold=15"
JAVA_OPTS+=" -XX:ParallelGCThreads=4"
JAVA_OPTS+=" -XX:+OptimizeStringConcat"

HEAP_DUMP_PATH="/heapDump/$( date '+%Y-%m-%d_%H-%M-%S' )_heap_dump.hprof"
JAVA_OPTS+=" -XX:+HeapDumpOnOutOfMemoryError -XX:HeapDumpPath=$HEAP_DUMP_PATH"
JAVA_OPTS+=" -XX:+ExitOnOutOfMemoryError"

# Extract Postgres database name and endpoint and ES host for multienvs
if [ -n "$MULTIENV_DB" -a -n "$MULTIENV_ES" -a -n "$MULTIENV_DB_CREDENTIALS" -a -n "$TARGET_ENV" ]; then
  echo "Migration running in multienv mode. Target env set to: $TARGET_ENV"
  MULTIENV_PG_HOST=$(echo $MULTIENV_DB | jq -Mr .$TARGET_ENV.clusterEndpoint)
  MULTIENV_PG_DB_NAME=$(echo $MULTIENV_DB | jq -Mr .$TARGET_ENV.databaseName)
  MULTIENV_PG_USER=$(echo $MULTIENV_DB_CREDENTIALS | jq -Mr .$TARGET_ENV.username)
  MULTIENV_PG_PASS=$(echo $MULTIENV_DB_CREDENTIALS | jq -Mr .$TARGET_ENV.password)
  MULTIENV_ES_HOST=$(echo $MULTIENV_ES | jq -Mr .$TARGET_ENV.clusterEndpoint)

  echo "Multienv PG_HOST: $MULTIENV_PG_HOST and DB_NAME: $MULTIENV_PG_DB_NAME"
  echo "Multienv PG_USER: $MULTIENV_PG_USER and PG_PASS: ${MULTIENV_PG_PASS:0:5}..."
  echo "Multienv ES_HOST: $MULTIENV_ES_HOST"

  export POSTGRES_HOST=$MULTIENV_PG_HOST
  export POSTGRES_DATABASE=$MULTIENV_PG_DB_NAME
  export POSTGRES_USER=$MULTIENV_PG_USER
  export POSTGRES_PASS=$MULTIENV_PG_PASS
  export CONNECTOR_MIGRATION_ES_HOST=$MULTIENV_ES_HOST
fi

DATASOURCE_URI=""
if [ -n "$POSTGRES_HOST" -a -n "$POSTGRES_DATABASE" ]; then
  DATASOURCE_URI="jdbc:postgresql://$POSTGRES_HOST"
  if [ -n "$POSTGRES_PORT" ]; then
    DATASOURCE_URI+=":$POSTGRES_PORT"
  fi

  DATASOURCE_URI+="/$POSTGRES_DATABASE"
fi

if [ -n "$DATASOURCE_URI" ]; then
  echo "Database URL set to: $DATASOURCE_URI"
  JAVA_OPTS+=" -Dspring.datasource.url=$DATASOURCE_URI"
fi

if [ -n "$POSTGRES_USER" -a -n "$POSTGRES_PASS" ]; then
  JAVA_OPTS+=" -Dspring.datasource.username=$POSTGRES_USER"
  JAVA_OPTS+=" -Dspring.datasource.password=$POSTGRES_PASS"
fi

if [ -n "$CONNECTOR_MIGRATION_ES_INDEX_NAME" ]; then
  echo "Migration ES connector index set to: $CONNECTOR_MIGRATION_ES_INDEX_NAME"
  JAVA_OPTS+=" -Dconnector.migration.es.indexName=$CONNECTOR_MIGRATION_ES_INDEX_NAME"
fi

if [ -n "$CONNECTOR_MIGRATION_MAKE_REPORT" ]; then
  JAVA_OPTS+=" -Dconnector.migration.makeReport=$CONNECTOR_MIGRATION_MAKE_REPORT"
fi

if [[ "${CONNECTOR_MIGRATION_ES_HOST}" =~ ".amazonaws.com" ]]; then
  JAVA_OPTS+=" -Dconnector.migration.es.protocol=https"
  JAVA_OPTS+=" -Dconnector.migration.es.port=443"
else
	JAVA_OPTS+=" -Dconnector.migration.es.protocol=http"
  JAVA_OPTS+=" -Dconnector.migration.es.port=9200"
fi

if [ -n "$MIGRATION_IMPL" ]; then
  echo "Migration should use ES implementation profile: $MIGRATION_IMPL"
  JAVA_OPTS+=" -Dspring.profiles.active=$MIGRATION_IMPL"
fi

if [ -n "$EXT_JAVA_OPTS" ]; then
  echo "External java options: $EXT_JAVA_OPTS"
  JAVA_OPTS+=" $EXT_JAVA_OPTS"
fi

echo "JAVA_OPTS: $JAVA_OPTS"

echo -e '\n\n==============\n\n\n'

JAR_NAME="connectors-migration.jar"
if [ "$MIGRATION_IMPL" == "es56" ]; then
    JAR_NAME="connectors-migration-es56.jar"
elif [ "$MIGRATION_IMPL" == "es68" ]; then
    JAR_NAME="connectors-migration-es68.jar"
fi

echo "Using jarfile with name $JAR_NAME"

cd "$MIGRATION_ROOT"
java \
    $JAVA_OPTS \
    -jar $JAR_NAME \
    "$@" &
PID=$!

EXITCODE=0
wait "$PID" || EXITCODE=$?
echo "Java process exited with $EXITCODE"
exit "$EXITCODE"
